@@ -1,5 +1,5 @@
 from .__version__ import __version__
-from .cleanups import add_cleanup, add_critical_cleanup
+from .cleanups import add_cleanup, add_critical_cleanup, add_success_only_cleanup
 from .conf import config
 from .ctx import context
 from .ctx import g, session, test
