@@ -104,7 +104,8 @@ def _cleanup_context():
             exc_info = sys.exc_info()
             raise
     finally:
-        call_cleanups(critical_only=exc_info is not None and exc_info[0] is KeyboardInterrupt)
+        call_cleanups(critical_only=exc_info is not None and exc_info[0] is KeyboardInterrupt,
+                      success_only=exc_info is None)
         del exc_info
 
 @contextmanager
@@ -164,8 +165,7 @@ def _set_current_test_context(test):
         context.test_methodname = test.__slash__.factory_name
     else:
         context.test_classname = test.__slash__.factory_name
-        # this includes a dot (.), so it has to be truncated
-        context.test_methodname = test.__slash__.address_in_factory[1:]
+        context.test_methodname = test.__slash__.address_in_factory
     try:
         yield
     finally:
