@@ -5,7 +5,7 @@ def test_autouse_fixtures_global(populated_suite):
     results = populated_suite.run().session.results
     assert len(results) > 0
     for result in results:
-        assert fixture.uuid in result.data['started_fixtures']
+        assert fixture.uuid in result.data['active_fixture_uuid_snapshot']
 
 def test_autouse_fixtures_specific_module(populated_suite, suite_test):
 
@@ -13,5 +13,4 @@ def test_autouse_fixtures_specific_module(populated_suite, suite_test):
     results = populated_suite.run().session.results
     assert len(results) > 0
     assert len(populated_suite.files) > 1
-    assert not all(fixture.uuid in result.data.get('started_fixtures', {})
-                   for result in results)
+    assert not all(fixture.uuid in result.data['active_fixture_uuid_snapshot'] for result in results)
